﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Microsoft.Research.Kinect.Nui;

using System.Runtime.InteropServices;
using System.Drawing.Imaging;

namespace Skeleton
{
    public partial class Form1 : Form
    {
        Runtime nui;
        PlanarImage videoimage;
        public Form1()
        {
            InitializeComponent();
            nui = Runtime.Kinects[0];
        }

        private void button1_Click(object sender, EventArgs e)
        {
            nui.Initialize(RuntimeOptions.UseDepthAndPlayerIndex | RuntimeOptions.UseSkeletalTracking | RuntimeOptions.UseColor);
            nui.VideoStream.Open(ImageStreamType.Video, 2, ImageResolution.Resolution640x480, ImageType.Color);
            //     nui.DepthStream.Open(ImageStreamType.Depth, 2, ImageResolution.Resolution320x240, ImageType.DepthAndPlayerIndex);
            nui.SkeletonFrameReady += new EventHandler<SkeletonFrameReadyEventArgs>(SkeletonFrameReady);
            nui.VideoFrameReady += new EventHandler<ImageFrameReadyEventArgs>(FrameReady);
        }

        void SkeletonFrameReady(object sender, SkeletonFrameReadyEventArgs e)
        {
            if (videoimage.Bits == null) return;
            SkeletonFrame skeletonFrame = e.SkeletonFrame;
            foreach (SkeletonData data in skeletonFrame.Skeletons)
            {
                if (data.TrackingState == SkeletonTrackingState.Tracked)
                {
                    Vector location = data.Joints[JointID.Head].Position;

                    float xd, yd;
                    nui.SkeletonEngine.SkeletonToDepthImage(location, out xd, out yd);
                    xd = Math.Max(0, Math.Min(xd * 320, 320));
                    yd = Math.Max(0, Math.Min(yd * 240, 240));

                    int x, y;
                    ImageViewArea iv = new ImageViewArea();
                    nui.NuiCamera.GetColorPixelCoordinatesFromDepthPixel(ImageResolution.Resolution640x480, iv, (int)xd, (int)yd, (short)0, out x, out y);
                    MarkAtxy(x, y);
                }
                Bitmap bmap = PImageToBitmap(videoimage);
                pictureBox1.Image = bmap;
            }

        }
        void FrameReady(object sender, ImageFrameReadyEventArgs e)
        {
            videoimage = e.ImageFrame.Image;

        }

        void MarkAtxy(int x, int y)
        {
            if (x < 4) return;
            if (x > (videoimage.Width - 4)) return;
            if (y < 4) return;
            if (y > (videoimage.Height - 4)) return;

            int i = indexOfPixelinBytes(x, y, videoimage.Width, videoimage.BytesPerPixel);

            int stride = videoimage.Width * videoimage.BytesPerPixel;
            for (int j = 0; j < 8; j++)
            {
                videoimage.Bits[i + (j - 3) * 4] = 0xFF;
                videoimage.Bits[i + (j - 3) * 4 + 1] = 0xFF;
                videoimage.Bits[i + (j - 3) * 4 + 2] = 0xFF;

                videoimage.Bits[i + (j - 3) * stride] = 0xFF;
                videoimage.Bits[i + (j - 3) * stride + 1] = 0xFF;
                videoimage.Bits[i + (j - 3) * stride + 2] = 0xFF;
            }


        }

        Bitmap PImageToBitmap(PlanarImage PImage)
        {
            Bitmap bmap = new Bitmap(
               PImage.Width,
               PImage.Height,
               PixelFormat.Format32bppRgb);
            BitmapData bmapdata = bmap.LockBits(
                new Rectangle(0, 0, PImage.Width,
                PImage.Height),
                ImageLockMode.WriteOnly,
                bmap.PixelFormat);
            IntPtr ptr = bmapdata.Scan0;
            Marshal.Copy(PImage.Bits,
                         0,
                         ptr,
                         PImage.Width *
                         PImage.BytesPerPixel *
                         PImage.Height);
            bmap.UnlockBits(bmapdata);
            return bmap;
        }
        int indexOfPixelinBytes(int x, int y, int width, int bpp)
        {
            return (x + y * width) * bpp;
        }
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            nui.Uninitialize();
        }

    }
}
